﻿//
// l3basic.cpp
//
#include "l3basic.h"
#include "configbox.h"
#include "wx/filename.h"
#include "wx/fontenum.h"
#include "res/l3basic.xpm"

#define ROM_BASIC_STRING "ROM BASIC"

#define DEFAULT_TEXTWIDTH 180

IMPLEMENT_APP(L3basicApp)

bool L3basicApp::OnInit()
{
	SetAppPath();
	SetAppName(_T("l3basic"));

	// load ini file
	mConfig.Load(GetAppPath() + GetAppName() + _T(".ini"));

	// set locale search path and catalog name
	mLocale.AddCatalogLookupPathPrefix(app_path + _T("lang"));
	mLocale.AddCatalogLookupPathPrefix(_T("lang"));
	mLocale.AddCatalog(_T("l3basic"));

	if (!wxApp::OnInit()) {
		return false;
	}

	L3basicFrame *frame = new L3basicFrame(GetAppName(), wxSize(640, 480) );
	frame->Show(true);
	SetTopWindow(frame);

	if (!frame->Init()) {
		return false;
	}

	return true;
}

int L3basicApp::OnExit()
{
	// save ini file
	mConfig.Save();

	return 0;
}

void L3basicApp::SetAppPath()
{
	app_path = wxFileName::FileName(argv[0]).GetPath(wxPATH_GET_SEPARATOR);
}

wxString L3basicApp::GetAppPath()
{
	return app_path;
}

Config *L3basicApp::GetConfig()
{
	return &mConfig;
}

//
// Frame
//
// Attach Event
BEGIN_EVENT_TABLE(L3basicFrame, wxFrame)
	// menu event
	EVT_MENU(wxID_EXIT,  L3basicFrame::OnQuit)
	EVT_MENU(IDM_ABOUT, L3basicFrame::OnAbout)

	EVT_MENU(IDM_OPEN_FILE, L3basicFrame::OnOpenFile)
	EVT_MENU(IDM_CLOSE_FILE, L3basicFrame::OnCloseFile)

	EVT_MENU(IDM_EXPORT_BASICBIN, L3basicFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_ASCIITXT, L3basicFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_UTF8TEXT, L3basicFrame::OnExportFile)

	EVT_MENU(IDM_CONFIGURE, L3basicFrame::OnConfigure)

	EVT_MENU_OPEN(L3basicFrame::OnMenuOpen)
END_EVENT_TABLE()

L3basicFrame::L3basicFrame(const wxString& title, const wxSize& size)
       : wxFrame(NULL, -1, title, wxDefaultPosition, size)
{
	// icon
#ifdef __WXMSW__
	SetIcon(wxIcon(_T("l3basic")));
#elif defined(__WXGTK__) || defined(__WXMOTIF__)
	SetIcon(wxIcon(l3basic_xpm));
#endif

	// menu
	menuFile = new wxMenu;
	menuOther = new wxMenu;
	menuHelp = new wxMenu;
	wxMenu *smenu;

	// file menu
	menuFile->Append( IDM_OPEN_FILE, _("&Open...") );
	menuFile->Append( IDM_CLOSE_FILE, _("&Close") );
	menuFile->AppendSeparator();
	smenu = new wxMenu;
	smenu->Append( IDM_EXPORT_BASICBIN, _("&BASIC Intermediate language...") );
	smenu->Append( IDM_EXPORT_ASCIITXT, _("&Ascii Text...") );
	smenu->Append( IDM_EXPORT_UTF8TEXT, _("&UTF-8 Text...") );
	menuFile->Append( IDM_EXPORT_FILE, _("&Export to"), smenu );
	menuFile->AppendSeparator();
	menuFile->Append( wxID_EXIT, _("E&xit") );
	// other menu
	menuOther->Append( IDM_CONFIGURE, _("&Configure...") );
	// help menu
	menuHelp->Append( IDM_ABOUT, _("&About...") );

	// menu bar
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, _("&File") );
	menuBar->Append( menuOther, _("&Other") );
	menuBar->Append( menuHelp, _("&Help") );

	SetMenuBar( menuBar );

	// control panel
	panel = new L3basicPanel(this);
	// config box
	cfgbox = new ConfigBox(this, IDD_CONFIGBOX);
	// char type box
	ctypebox = new CharTypeBox(this, IDD_CHARTYPEBOX);
}

L3basicFrame::~L3basicFrame()
{
	// save ini file
	Config *ini = wxGetApp().GetConfig();
	ini->SetFilePath(file_path);
	ini->SetNewLineAscii(ps->GetNewLineAscii());
	ini->SetNewLineUtf8(ps->GetNewLineUtf8());
	ini->SetIncludeBOM(ps->GetIncludeBOM());
	ini->SetStartAddr(ps->GetStartAddr());

	delete ctypebox;
	delete cfgbox;
	delete ps;
}

bool L3basicFrame::Init()
{
	// initialize
	ps = new Parse(wxGetApp().GetAppPath());
	if (!ps->Init()) {
		return false;
	}
	// load ini file
	Config *ini = wxGetApp().GetConfig();
	file_path = ini->GetFilePath();
	ps->SetNewLineAscii(ini->GetNewLineAscii());
	ps->SetNewLineUtf8(ini->GetNewLineUtf8());
	ps->SetIncludeBOM(ini->GetIncludeBOM());
	ps->SetStartAddr(ini->GetStartAddr());

	// set combo box on panel
	wxArrayString basic_types;
	ps->GetBasicTypes(basic_types);
	panel->AddBasicType(basic_types);
	wxArrayString char_types;
	ps->GetCharTypes(char_types);
	panel->AddCharType(char_types);
	panel->Update(NULL);

	// set combo box on configbox
	int *items;
	size_t count;
	items = ps->GetStartAddrsPtr(&count);
	cfgbox->AddStartAddrItems(items, count);

	// set combo box on char type box
	ctypebox->AddCharType(char_types);

	return true;
}

// メニュー更新
void L3basicFrame::OnMenuOpen(wxMenuEvent& event)
{
	wxMenu *menu = event.GetMenu();

	if (menu == NULL) return;

	if (menu == menuFile) {	// File...
		// Export to ...
		menu->Enable(IDM_CLOSE_FILE,  false);
		menu->Enable(IDM_EXPORT_BASICBIN, false);
		menu->Enable(IDM_EXPORT_ASCIITXT, false);
		menu->Enable(IDM_EXPORT_UTF8TEXT, false);

		if (ps->IsOpenedDataFile()) {
			menu->Enable(IDM_CLOSE_FILE, true);
			PsFileType *flags = ps->GetOpenedDataTypePtr();
			if (flags->GetFlag(psAscii)) {
				// ascii
				menu->Enable(IDM_EXPORT_BASICBIN, true);
				menu->Enable(IDM_EXPORT_ASCIITXT, true);
				menu->Enable(IDM_EXPORT_UTF8TEXT, true);
			} else {
				// binary
				menu->Enable(IDM_EXPORT_ASCIITXT, true);
				menu->Enable(IDM_EXPORT_UTF8TEXT, true);
			}
		}
	}
}
// ドロップされたファイルを開く
void L3basicFrame::OpenDroppedFile(wxString &path)
{
	CloseDataFile();
	OpenDataFile(path);
}

void L3basicFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
 	CloseDataFile();
	Close(true);
}

void L3basicFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	L3basicAbout(this, wxID_ANY).ShowModal();
}

void L3basicFrame::OnOpenFile(wxCommandEvent& WXUNUSED(event))
{
	L3basicFileDialog *dlg = new L3basicFileDialog(
		_("Open file"),
		file_path,
		wxEmptyString,
		_("Supported files (*.bin;*.bas;*.txt;*.dat)|*.bin;*.bas;*.txt;*.dat|All files (*.*)|*.*"),
		wxFD_OPEN);

	int rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		OpenDataFile(path);
	}
}

void L3basicFrame::OnCloseFile(wxCommandEvent& WXUNUSED(event))
{
	CloseDataFile();
}

// エクスポート
void L3basicFrame::OnExportFile(wxCommandEvent& event)
{
	int id = event.GetId();

	ExportFile(id);
}

// 設定ダイアログ
void L3basicFrame::OnConfigure(wxCommandEvent& WXUNUSED(event))
{
	cfgbox->SetNewLineAscii(ps->GetNewLineAscii());
	cfgbox->SetNewLineUtf8(ps->GetNewLineUtf8());
	cfgbox->SetIncludeBOM(ps->GetIncludeBOM());
	cfgbox->SetStartAddr(ps->GetStartAddr());

	cfgbox->ShowModal();

	ps->SetNewLineAscii(cfgbox->GetNewLineAscii());
	ps->SetNewLineUtf8(cfgbox->GetNewLineUtf8());
	ps->SetIncludeBOM(cfgbox->GetIncludeBOM());
	ps->SetStartAddr(cfgbox->GetStartAddr());
}

// 指定したファイルを開く
void L3basicFrame::OpenDataFile(wxString &path)
{
	wxString title;

	// set recent file path
	file_path = wxFileName::FileName(path).GetPath(wxPATH_GET_SEPARATOR);

	if (!ps->OpenDataFile(path)) {
		return;
	}

	// update window
	title = wxGetApp().GetAppName() + _T(" - ") + path;
	SetTitle(title);
	// update panel
	panel->Update(ps->GetOpenedDataTypePtr());
	panel->SetBasicType(ps->GetOpenedBasicType());
	panel->GetTextName()->SetValue(path);

	wxArrayString lines;
	wxString char_type = ps->GetParsedData(lines);
	panel->SetTextInfo(char_type, lines);
}

// ファイルを閉じる
void L3basicFrame::CloseDataFile()
{
	ps->CloseDataFile();

	// update window
	wxString title = wxGetApp().GetAppName();
	SetTitle(title);
	// update panel
	panel->Update(NULL);
	panel->GetTextName()->SetValue(_(""));
}

// エクスポート
void L3basicFrame::ExportFile(int id)
{
	int rc;
	wxString file_base;
	wxString wild_card;
	PsFileType file_type;

	if (!ps->IsOpenedDataFile()) return;

	file_base = ps->GetFileNameBase();

	PsFileType *opened_flags = ps->GetOpenedDataTypePtr();
	bool enable = true;

	switch(id) {
		case IDM_EXPORT_BASICBIN:
			file_base += _T(".bin");
			wild_card = _("BASIC file (*.bin)|*.bin|All files (*.*)|*.*");
			file_type.SetFlag(psBinary, true);
			if (!opened_flags->GetFlag(psAscii)) enable = false;
			break;
		case IDM_EXPORT_ASCIITXT:
			file_base += _T(".bas");
			wild_card = _("ASCII file (*.bas)|*.bas|All files (*.*)|*.*");
			file_type.SetFlag(psAscii, true);
			break;
		case IDM_EXPORT_UTF8TEXT:
			file_base += _T(".txt");
			wild_card = _("TEXT file (*.txt)|*.txt|All files (*.*)|*.*");
			file_type.SetFlag(psAscii | psUTF8, true);
			// show char type box
			rc = ctypebox->ShowModal();
			file_type.SetCharType(ctypebox->GetCharType());
			break;
		default:
			file_base += _T("");
			wild_card = _("All files (*.*)|*.*");
			file_type.SetFlag(psBinary, true);
			break;
	}
	if (!enable) return;

	L3basicFileDialog *dlg = new L3basicFileDialog(
		_("Export file"),
		file_path,
		file_base,
		wild_card,
		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);

	rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		wxString basic_type = panel->GetBasicType();
		file_type.SetBasicType(basic_type);
		if (!ps->OpenOutFile(path, file_type)) {
			return;
		}
		ps->ExportData();
		ps->CloseOutFile();

		// output report
		wxArrayString nlines;
		wxString nchar_type = ps->GetParsedData(nlines);
		panel->SetTextInfo(nchar_type, nlines);
	}
}

// ファイルを読み直す
void L3basicFrame::ReloadBinaryData(int type, int mask, const wxString &basic_type)
{
	if (ps->ReloadOpenedBinaryData(type, mask, basic_type)) {
		wxArrayString nlines;
		wxString nchar_type = ps->GetParsedData(nlines);
		panel->SetTextInfo(nchar_type, nlines);
	}
}

// ファイルを読み直す
void L3basicFrame::ReloadOpendData(int type, int mask, const wxString &char_type)
{
	if (ps->ReloadOpendAsciiData(type, mask, char_type)) {
		wxArrayString nlines;
		wxString nchar_type = ps->GetParsedData(nlines);
		panel->SetTextInfo(nchar_type, nlines);
	}
}

// ファイルを読み直す
void L3basicFrame::ReloadParsedData(int type, int mask, const wxString &char_type)
{
	if (ps->ReloadParsedData(type, mask, char_type)) {
		wxArrayString nlines;
		wxString nchar_type = ps->GetParsedData(nlines);
		panel->SetTextInfo(nchar_type, nlines);
	}
}

//
// Control Panel
//
// Attach Event
BEGIN_EVENT_TABLE(L3basicPanel, wxPanel)
	// event
	EVT_SIZE(L3basicPanel::OnSize)

	EVT_COMBOBOX(IDC_COMBO_BASICTYPE, L3basicPanel::OnSelectBasicType)

	EVT_RADIOBUTTON(IDC_RADIO_ASCII, L3basicPanel::OnSelectAscii)
	EVT_RADIOBUTTON(IDC_RADIO_UTF8, L3basicPanel::OnSelectUTF8)

	EVT_COMBOBOX(IDC_COMBO_CHARTYPE, L3basicPanel::OnSelectCharType)
	EVT_COMBOBOX(IDC_COMBO_DCHARTYPE, L3basicPanel::OnSelectDCharType)

	EVT_BUTTON(IDC_BUTTON_FONT, L3basicPanel::OnClickFont)
END_EVENT_TABLE()

L3basicPanel::L3basicPanel(L3basicFrame *parent)
       : wxPanel(parent)
{
	frame    = parent;
	textName = NULL;
	textInfo = NULL;

	wxSize  frame_size = parent->GetClientSize();
	long style;

	wxSizerFlags flagsM = wxSizerFlags().Expand().Border(wxALL, 0);
	wxSizerFlags flagsW = wxSizerFlags().Expand().Border(wxALL, 2);
	wxBoxSizer *hbox;
	wxSize size;
	size.x = DEFAULT_TEXTWIDTH; size.y = -1;

	szrAll = new wxBoxSizer(wxVERTICAL);

	wxBoxSizer *vbox = new wxStaticBoxSizer(new wxStaticBox(this, wxID_ANY, _("Input file information")), wxVERTICAL);

	style = wxTE_READONLY;
	textName = new wxTextCtrl(this, IDC_TEXT_NAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, style);
	vbox->Add(textName, flagsW);

	hbox = new wxBoxSizer(wxHORIZONTAL);
	radBinary   = new wxRadioButton(this, IDC_RADIO_BINARY, _("BASIC Intermediate language"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	hbox->Add(radBinary, flagsW);
	hbox->Add(new wxStaticText(this, wxID_ANY, _T("    ")), flagsW);
	hbox->Add(new wxStaticText(this, wxID_ANY, _("BASIC type")), flagsW);
	comBasicType = new wxComboBox(this, IDC_COMBO_BASICTYPE, _T(""), wxDefaultPosition, size, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY);
	hbox->Add(comBasicType, flagsW);
	vbox->Add(hbox, flagsW);

	hbox = new wxBoxSizer(wxHORIZONTAL);
	radAscii = new wxRadioButton(this, IDC_RADIO_ASCII, _("Ascii text"));
	radUTF8  = new wxRadioButton(this, IDC_RADIO_UTF8 , _("UTF-8 text"));
	comCharType = new wxComboBox(this, IDC_COMBO_CHARTYPE, _T(""), wxDefaultPosition, size, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY);
	hbox->Add(radAscii, flagsW);
	hbox->Add(radUTF8, flagsW);
	hbox->Add(comCharType, flagsW);
	vbox->Add(hbox, flagsW);

	szrAll->Add(vbox, flagsW);

	wxBoxSizer *vboxb = new wxBoxSizer(wxVERTICAL);
	hbox = new wxBoxSizer(wxHORIZONTAL);
	hbox->Add(new wxStaticText(this, wxID_ANY, _("Display type")), flagsW);
	comDCharType = new wxComboBox(this, IDC_COMBO_DCHARTYPE, _T(""), wxDefaultPosition, size, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY);
	hbox->Add(comDCharType, flagsW);
	hbox->Add(new wxStaticText(this, wxID_ANY, _T("  ")), flagsW);
	hbox->Add(new wxStaticText(this, wxID_ANY, _("Font")), flagsW);
	style = wxTE_READONLY;
	textFont = new wxTextCtrl(this, IDC_TEXT_FONT, wxEmptyString, wxDefaultPosition, size, style);
	btnFont = new wxButton(this, IDC_BUTTON_FONT, _("Change"));
	hbox->Add(textFont, flagsW);
	hbox->Add(btnFont, flagsW);
	vboxb->Add(hbox, flagsW);

	szrAll->Add(vboxb, flagsM);

	wxBoxSizer *vboxc = new wxBoxSizer(wxVERTICAL);
	style = wxTE_READONLY | wxTE_MULTILINE;
	size.x = frame_size.x; size.y = -1;
	textInfo = new wxTextCtrl(this, IDC_TEXT_INFO, wxEmptyString, wxDefaultPosition, size, style);
	hbox = new wxBoxSizer(wxHORIZONTAL);
	hbox->Add(textInfo, flagsM);
	vboxc->Add(hbox, flagsM);

	szrAll->Add(vboxc, flagsM);
	szrAll->SetSizeHints(this);

	SetSizerAndFit(szrAll);
	Layout();

	// drag and drop
	textName->SetDropTarget(new L3basicFileDropTarget(parent));
	textInfo->SetDropTarget(new L3basicFileDropTarget(parent));

	// set font name
	SetInitialFont();
	SetTextFontName();
	textInfo->SetFont(fontFixed);
}

L3basicPanel::~L3basicPanel()
{
	// save ini file
	Config *ini = wxGetApp().GetConfig();
	ini->SetFontName(mFontName);
	ini->SetFontSize(mFontSize);
}

// リサイズ
void L3basicPanel::OnSize(wxSizeEvent& event)
{
	wxSize size = event.GetSize();

//	if (textName) textName->SetSize(size.x, -1);
	if (textInfo) textInfo->SetSize(size.x, size.y - textInfo->GetPosition().y);
}

// BASIC種類選択
void L3basicPanel::OnSelectBasicType(wxCommandEvent& event)
{
	frame->ReloadBinaryData(psBinary, psAscii, GetBasicType());
}
// Asciiラジオボタン選択
void L3basicPanel::OnSelectAscii(wxCommandEvent& event)
{
	frame->ReloadOpendData(psAscii, psUTF8, _T(""));
}
// UTF8ラジオボタン選択
void L3basicPanel::OnSelectUTF8(wxCommandEvent& event)
{
	frame->ReloadOpendData(psAscii | psUTF8, 0, GetCharType(0));
}
// 文字種類選択
void L3basicPanel::OnSelectCharType(wxCommandEvent& event)
{
	frame->ReloadOpendData(psAscii | psUTF8, 0, GetCharType(0));
}
// 表示する文字種類選択
void L3basicPanel::OnSelectDCharType(wxCommandEvent& event)
{
	frame->ReloadParsedData(psAscii | psUTF8, 0, GetCharType(1));
}
// フォントボタン選択
void L3basicPanel::OnClickFont(wxCommandEvent& event)
{
#ifdef USE_FONTDIALOG
	wxFontData fdata;

	// font dialog
	fdata.SetInitialFont(fontFixed);
	wxFontDialog *fontdlg = new wxFontDialog(this, fdata);

	if (fontdlg->ShowModal() == wxID_OK) {

		fdata = fontdlg->GetFontData();
		fontFixed = fdata.GetChosenFont();

		mFontName = fontFixed.GetFaceName();
		mFontSize = fontFixed.GetPointSize();

		// set font
		SetTextFontName();
		textInfo->SetFont(fontFixed);
	}

	delete fontdlg;
#else
	FontMiniBox fontbox(this, wxID_ANY);
	fontbox.SetFontName(mFontName);
	fontbox.SetFontSize(mFontSize);

	if (fontbox.ShowModal() == wxID_OK) {
		wxFont new_font = wxFont(fontbox.GetFontSize(), wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false, fontbox.GetFontName());
		if (new_font.IsOk()) {
			mFontName = new_font.GetFaceName();
			mFontSize = new_font.GetPointSize();
			fontFixed = new_font;
		}

		// set font
		SetTextFontName();
		textInfo->SetFont(fontFixed);
	}
#endif
}

// コントロールを更新
void L3basicPanel::Update(PsFileType *file_type)
{
	radBinary->Enable(false);
	radAscii->Enable(false);
	radUTF8->Enable(false);
	comCharType->Enable(false);

	if (file_type == NULL) return;

	if (file_type->GetFlag(psAscii)) {
		radAscii->Enable(true);
		radUTF8->Enable(true);
		comCharType->Enable(true);
		if (file_type->GetFlag(psUTF8)) {
			// infile is ascii utf8
			radUTF8->SetValue(true);
			SetCharType(0, file_type->GetCharType());
		} else {
			// infile is ascii text
			radAscii->SetValue(true);
		}
	} else {
		// infile is binary
		radBinary->Enable(true);
		radBinary->SetValue(true);
	}
}

// Basic type を更新
void L3basicPanel::AddBasicType(const wxArrayString &items)
{
	mOrigBasicTypes.Empty();
	comBasicType->Clear();
	for(size_t i=0; i<items.GetCount(); i++) {
		mOrigBasicTypes.Add(items[i]);
		if (items[i] == _T("")) {
			comBasicType->Insert(_(ROM_BASIC_STRING), i);
		} else {
			comBasicType->Insert(wxGetTranslation(items[i]), i);
		}
	}
	comBasicType->Select(0);
}
// Basic type を返す
wxString L3basicPanel::GetBasicType()
{
	return mOrigBasicTypes[comBasicType->GetSelection()];
}
// Basic typeをセット
void L3basicPanel::SetBasicType(int pos)
{
	comBasicType->Select(pos);
}
void L3basicPanel::SetBasicType(const wxString &basic_type)
{
	size_t i = 0;
	for(; i<mOrigBasicTypes.GetCount(); i++) {
		if (mOrigBasicTypes[i] == basic_type) {
			break;
		}
	}
	if (i >= mOrigBasicTypes.GetCount()) {
		i = 0;
	}
	SetBasicType(i);
}

// Char type を更新
void L3basicPanel::AddCharType(const wxArrayString &items)
{
	mOrigCharTypes.Empty();
	comCharType->Clear();
	comDCharType->Clear();
	for(size_t i=0; i<items.GetCount(); i++) {
		mOrigCharTypes.Add(items[i]);
		comCharType->Insert(wxGetTranslation(items[i]), i);
		comDCharType->Insert(wxGetTranslation(items[i]), i);
	}
	comCharType->Select(0);
	comDCharType->Select(0);
}
// Char type を返す
wxString L3basicPanel::GetCharType(int n)
{
	wxComboBox *cb = comCharType;
	if (n == 1) {
		cb = comDCharType;
	}
	return mOrigCharTypes[cb->GetSelection()];
}
// Char typeをセット
void L3basicPanel::SetCharType(int n, int pos)
{
	wxComboBox *cb = comCharType;
	if (n == 1) {
		cb = comDCharType;
	}
	cb->Select(pos);
}
void L3basicPanel::SetCharType(int n, const wxString &char_type)
{
	size_t i = 0;
	for(; i<mOrigCharTypes.GetCount(); i++) {
		if (mOrigCharTypes[i] == char_type) {
			break;
		}
	}
	if (i >= mOrigCharTypes.GetCount()) {
		i = 0;
	}
	SetCharType(n, i);
}

// 情報をセット
void L3basicPanel::SetTextInfo(const wxString &char_type)
{
	SetCharType(1, char_type);
}
void L3basicPanel::SetTextInfo(const wxString &char_type, const wxString &str)
{
	SetTextInfo(char_type);
	textInfo->SetValue(str);
}
void L3basicPanel::SetTextInfo(const wxString &char_type, const wxArrayString &lines)
{
	SetTextInfo(char_type);

	wxString str;

	for(size_t i=0; i<lines.GetCount(); i++) {
		str += lines[i] + _T("\n");
	}
	textInfo->SetValue(str);
}
// フォント名を表示
void L3basicPanel::SetTextFontName()
{
	wxString point_str;

	point_str.Printf(_T(" (%dpt)"), mFontSize);

	textFont->SetValue(mFontName + point_str);
}
// フォントを設定
void L3basicPanel::SetInitialFont()
{
	// load ini file
	Config *ini = wxGetApp().GetConfig();
	mFontName = ini->GetFontName();
	mFontSize = ini->GetFontSize();

	if (mFontName.IsEmpty()) {
		fontFixed = textInfo->GetFont();
	} else {
		fontFixed = wxFont(mFontSize, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL, false, mFontName);
	}
	if (!fontFixed.IsOk()) {
		fontFixed = wxFont();
	}
	mFontName = fontFixed.GetFaceName();
	mFontSize = fontFixed.GetPointSize();
}

//
// Char type box
//
// Attach Event
BEGIN_EVENT_TABLE(CharTypeBox, wxDialog)
END_EVENT_TABLE()

CharTypeBox::CharTypeBox(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("Charactor type"), wxDefaultPosition, wxDefaultSize, wxCAPTION)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);
	wxSize size;

	wxBoxSizer *szrAll = new wxBoxSizer(wxVERTICAL);
	szrAll->Add(new wxStaticText(this, wxID_ANY, _("Choose charactor type.")), flags);
	size.x = DEFAULT_TEXTWIDTH; size.y = -1;
	comCharType = new wxComboBox(this, IDC_COMBO_CHARTYPE, wxEmptyString, wxDefaultPosition, size, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY);
	szrAll->Add(comCharType, flags);
	wxSizer *szrButtons = CreateButtonSizer(wxOK);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);

	mSelectedPos = 0;
}

int CharTypeBox::ShowModal()
{
	init_dialog();
	int rc = wxDialog::ShowModal();
	if (rc == wxID_OK) {
		term_dialog();
	}
	return rc;
}

void CharTypeBox::term_dialog()
{
	mSelectedPos = comCharType->GetSelection();
}

void CharTypeBox::AddCharType(const wxArrayString &items)
{
	mOrigCharTypes.Empty();
	comCharType->Clear();
	for(size_t i=0; i<items.GetCount(); i++) {
		mOrigCharTypes.Add(items[i]);
		comCharType->Insert(wxGetTranslation(items[i]), i);
	}
	comCharType->Select(0);
}
// Char type を返す
wxString CharTypeBox::GetCharType()
{
	return mOrigCharTypes[mSelectedPos];
}
// Char typeをセット
void CharTypeBox::SetCharType(int pos)
{
	mSelectedPos = pos;
	comCharType->Select(pos);
}
void CharTypeBox::SetCharType(const wxString &char_type)
{
	size_t i = 0;
	for(; i<mOrigCharTypes.GetCount(); i++) {
		if (mOrigCharTypes[i] == char_type) {
			break;
		}
	}
	if (i >= mOrigCharTypes.GetCount()) {
		i = 0;
	}
	SetCharType(i);
}

//
// Font Box
//
// Attach Event
BEGIN_EVENT_TABLE(FontMiniBox, wxDialog)
	EVT_TEXT(IDC_COMBO_FONTSIZE, FontMiniBox::OnTextSize)
END_EVENT_TABLE()

FontMiniBox::FontMiniBox(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("Font"), wxDefaultPosition, wxDefaultSize, wxCAPTION)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);
	wxSize size;

	wxTextValidator tVali(wxFILTER_NUMERIC);

	wxBoxSizer *szrAll = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	size.x = DEFAULT_TEXTWIDTH; size.y = -1;
	comFontName = new wxComboBox(this, IDC_COMBO_FONTNAME, wxEmptyString, wxDefaultPosition, size, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY);
	comFontSize = new wxComboBox(this, IDC_COMBO_FONTSIZE, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN, tVali);
	hbox->Add(comFontName, flags);
	hbox->Add(comFontSize, flags);
	szrAll->Add(hbox, flags);
	wxSizer *szrButtons = CreateButtonSizer(wxOK | wxCANCEL);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);
}

int FontMiniBox::ShowModal()
{
	init_dialog();
	int rc = wxDialog::ShowModal();
	if (rc == wxID_OK) {
		term_dialog();
	}
	return rc;
}

void FontMiniBox::init_dialog()
{
	mFontNames = wxFontEnumerator::GetFacenames();
	mFontNames.Sort();

	wxString size;
	mFontSizes.Empty();
	for(int i=4; i<=24; i++) {
		size.Printf(_T("%d"), i);
		mFontSizes.Add(size);
	}

	comFontName->Clear();
	comFontName->Insert(mFontNames, 0);
	comFontSize->Clear();
	comFontSize->Insert(mFontSizes, 0);

	comFontName->SetValue(mSelectedName);
	size.Printf(_T("%d"), mSelectedSize);
	comFontSize->SetValue(size);
}

void FontMiniBox::term_dialog()
{
	mSelectedName = comFontName->GetValue();
	long val;
	comFontSize->GetValue().ToLong(&val);
	if (val < 1) val = 1;
	if (val > 99) val = 99;
	mSelectedSize = (int)val;
}

void FontMiniBox::OnTextSize(wxCommandEvent& event)
{
	wxString mInputSize = event.GetString().Left(2);
	comFontSize->ChangeValue(mInputSize);
}

//
// File Dialog
//
L3basicFileDialog::L3basicFileDialog(const wxString& message, const wxString& defaultDir, const wxString& defaultFile, const wxString& wildcard, long style)
            : wxFileDialog(NULL, message, defaultDir, defaultFile, wildcard, style)
{
}

//
// File Drag and Drop
//
L3basicFileDropTarget::L3basicFileDropTarget(L3basicFrame *parent)
			: frame(parent)
{
}

bool L3basicFileDropTarget::OnDropFiles(wxCoord x, wxCoord y ,const wxArrayString &filenames)
{
	if (filenames.Count() > 0) {
		wxString name = filenames.Item(0);
		frame->OpenDroppedFile(name);
	}
    return true;
}

//
// About dialog
//
L3basicAbout::L3basicAbout(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("About..."), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);

	wxBoxSizer *szrLeft   = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrRight  = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrMain   = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *szrAll    = new wxBoxSizer(wxVERTICAL);

	szrLeft->Add(new wxStaticBitmap(this, wxID_ANY,
		wxBitmap(l3basic_xpm), wxDefaultPosition, wxSize(64, 64))
		, flags);

	wxString str = _T("");
	str += _T("L3basic, Version ");
	str += _T(APPLICATION_VERSION);
	str += _T(" \"");
	str += _T(PLATFORM);
	str += _T("\"\n\n");
	str	+= _T("using ");
	str += wxVERSION_STRING;
	str += _T("\n\n");
	str	+= _T("Copyright (C) 2013 Sasaji");

	szrRight->Add(new wxStaticText(this, wxID_ANY, str), flags);

	wxSizer *szrButtons = CreateButtonSizer(wxOK);
	szrMain->Add(szrLeft, flags);
	szrMain->Add(szrRight, flags);
	szrAll->Add(szrMain, flags);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);
}
