//
// l3basic.h
//
#ifndef _L3BASIC_H_
#define _L3BASIC_H_

#include "common.h"
#include "wx/wx.h"
#include "wx/dynarray.h"
#include "wx/dnd.h"
#include "wx/fontdlg.h"
#include "config.h"
#include "parse.h"

class L3basicApp;
class L3basicFrame;
class L3basicPanel;
class L3basicFileDialog;
class L3basicFileDropTarget;

class CharTypeBox;

class ConfigBox;

// Window
class L3basicApp: public wxApp
{
private:
	wxString app_path;
	wxLocale mLocale;
	Config   mConfig;

	void SetAppPath();
public:
	L3basicApp() : mLocale(wxLANGUAGE_DEFAULT) {}
	bool OnInit();
	int  OnExit();
	wxString GetAppPath();

	Config *GetConfig();
};

DECLARE_APP(L3basicApp)

// Frame
class L3basicFrame: public wxFrame
{
private:
	// gui
	wxMenu *menuFile;
	wxMenu *menuOther;
	wxMenu *menuHelp;
	L3basicPanel *panel;

	Parse *ps;

	ConfigBox *cfgbox;
	CharTypeBox *ctypebox;

	wxString file_path;

public:

    L3basicFrame(const wxString& title, const wxSize& size);
	~L3basicFrame();

	bool Init();

	// event procedures
	void OnQuit(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);

	void OnOpenFile(wxCommandEvent& event);
	void OnCloseFile(wxCommandEvent& event);

	void OnExportFile(wxCommandEvent& event);

	void OnConfigure(wxCommandEvent& event);

	void OnMenuOpen(wxMenuEvent& event);

	// functions
	void OpenDataFile(wxString &path);
	void CloseDataFile();
	void ExportFile(int id);
	void OpenDroppedFile(wxString &path);
	void ReloadBinaryData(int type, int mask, const wxString &basic_type);
	void ReloadOpendData(int type, int mask, const wxString &char_type);
	void ReloadParsedData(int type, int mask, const wxString &char_type);

	// properties
	L3basicPanel *GetL3basicPanel() { return panel; }

	enum
	{
		// menu id
		IDM_EXIT = 1,
		IDM_ABOUT,
		IDM_OPEN_FILE,
		IDM_CLOSE_FILE,
		IDM_EXPORT_FILE,
		IDM_EXPORT_BASICBIN,
		IDM_EXPORT_ASCIITXT,
		IDM_EXPORT_UTF8TEXT,
		IDM_CONFIGURE,

		IDD_CONFIGBOX,
		IDD_CHARTYPEBOX,
	};

	DECLARE_EVENT_TABLE()
};

// Panel
class L3basicPanel: public wxPanel
{
private:
	L3basicFrame *frame;

	wxBoxSizer *szrAll;

	wxTextCtrl *textName;
	wxTextCtrl *textInfo;

	wxRadioButton *radBinary;
	wxRadioButton *radAscii;
	wxRadioButton *radUTF8;

	wxComboBox *comBasicType;
	wxComboBox *comCharType;

	wxComboBox *comDCharType;

	wxTextCtrl *textFont;
	wxButton *btnFont;

	wxString mFontName;
	int      mFontSize;
	wxFont fontFixed;

	wxArrayString mOrigBasicTypes;
	wxArrayString mOrigCharTypes;

	void SetInitialFont();
public:
	L3basicPanel(L3basicFrame *parent);
	~L3basicPanel();

	// event procedures
	void OnSize(wxSizeEvent& event);
	void OnSelectBasicType(wxCommandEvent& event);
	void OnSelectAscii(wxCommandEvent& event);
	void OnSelectUTF8(wxCommandEvent& event);
	void OnSelectCharType(wxCommandEvent& event);
	void OnSelectDCharType(wxCommandEvent& event);
	void OnClickFont(wxCommandEvent& event);

	// functions
	void Update(PsFileType *file_type);
	void AddBasicType(const wxArrayString &items);
	wxString GetBasicType();
	void SetBasicType(int pos);
	void SetBasicType(const wxString &basic_type);

	void AddCharType(const wxArrayString &items);
	wxString GetCharType(int n);
	void SetCharType(int n, int pos);
	void SetCharType(int n, const wxString &char_type);

	void SetTextInfo(const wxString &char_type);
	void SetTextInfo(const wxString &char_type, const wxString &str);
	void SetTextInfo(const wxString &char_type, const wxArrayString &lines);
	void SetTextFontName();

	// properties
	wxTextCtrl *GetTextName() { return textName; }
	wxTextCtrl *GetTextInfo() { return textInfo; }

	void SetFontName(const wxString &val) { mFontName = val; }
	void SetFontSize(int val) { mFontSize = val; }
	wxString &GetFontName() { return mFontName; }
	int GetFontSize() { return mFontSize; }

	enum {
		IDC_TEXT_NAME = 1,
		IDC_TEXT_INFO,
		//
		IDC_RADIO_BINARY,
		IDC_RADIO_ASCII,
		IDC_RADIO_UTF8,
		IDC_COMBO_BASICTYPE,
		IDC_COMBO_CHARTYPE,

		IDC_COMBO_DCHARTYPE,

		IDC_TEXT_FONT,
		IDC_BUTTON_FONT
	};

	DECLARE_EVENT_TABLE()
};

// for char type box
class CharTypeBox : public wxDialog
{
private:
	wxComboBox *comCharType;

	wxArrayString mOrigCharTypes;

	int mSelectedPos;
	// parameter

	void init_dialog() {};
	void term_dialog();

public:
	CharTypeBox(wxWindow* parent, wxWindowID id);

	enum {
		IDC_COMBO_CHARTYPE = 1
	};

	// functions
	int ShowModal();
	void AddCharType(const wxArrayString &items);
	wxString GetCharType();
	void SetCharType(int pos);
	void SetCharType(const wxString &char_type);

	// event procedures

	// properties

	DECLARE_EVENT_TABLE()
};

// for font box
class FontMiniBox : public wxDialog
{
private:
	wxComboBox *comFontName;
	wxComboBox *comFontSize;

	wxArrayString mFontNames;
	wxArrayString mFontSizes;

	wxString mSelectedName;
	int      mSelectedSize;

	// parameter

	void init_dialog();
	void term_dialog();

public:
	FontMiniBox(wxWindow* parent, wxWindowID id);

	enum {
		IDC_COMBO_FONTNAME = 1,
		IDC_COMBO_FONTSIZE
	};

	// functions
	int ShowModal();

	// event procedures
	void OnTextSize(wxCommandEvent& event);

	// properties
	wxString &GetFontName() { return mSelectedName; }
	int GetFontSize() { return mSelectedSize; }
	void SetFontName(const wxString &val) { mSelectedName = val; }
	void SetFontSize(int val) { mSelectedSize = val; }

	DECLARE_EVENT_TABLE()
};

// for file dialog
class L3basicFileDialog: public wxFileDialog
{
public:
	L3basicFileDialog(const wxString& message, const wxString& defaultDir = wxEmptyString, const wxString& defaultFile = wxEmptyString, const wxString& wildcard = wxFileSelectorDefaultWildcardStr, long style = wxFD_DEFAULT_STYLE);

};

// for drop
class L3basicFileDropTarget : public wxFileDropTarget
{
    L3basicFrame *frame;
public:
    L3basicFileDropTarget(L3basicFrame *parent);
    bool OnDropFiles(wxCoord x, wxCoord y ,const wxArrayString &filenames);
};

// About dialog
class L3basicAbout : public wxDialog
{
public:
	L3basicAbout(wxWindow* parent, wxWindowID id);
};

#endif /* _L3BASIC_H_ */

