﻿//
// マッピングテーブル
//
#include "maptable.h"
#include "wx/arrimpl.cpp"

// マッピングテーブルItem
CodeMapItem::CodeMapItem() {
	code = 0;
	str.Empty();
	str_upper.Empty();
	attr.Empty();
	bytes = NULL;
	bytes_length = 0;
}
CodeMapItem::CodeMapItem(const long &new_code, const wxString &new_str, const wxString &new_attr) {
	code = new_code;
	str = new_str;
	str_upper = new_str.Upper();
	attr = new_attr;

	wxCharBuffer buf = str.mb_str(wxConvUTF8);
	bytes_length = strlen(buf);
	bytes = new wxUint8[bytes_length+1];
	memset((void *)bytes, 0, bytes_length+1);
	memcpy((void *)bytes, (const void *)buf, bytes_length);
}
CodeMapItem::~CodeMapItem() {
	delete[] bytes;
}
CodeMapItem &CodeMapItem::operator=(const CodeMapItem &item) {
	code = item.code;
	str = item.str;
	str_upper = item.str.Upper();
	attr = item.attr;
	// duplicate
	bytes_length = strlen((const char *)item.bytes);
	bytes = new wxUint8[bytes_length+1];
	memset((void *)bytes, 0, bytes_length+1);
	memcpy((void *)bytes, (const void *)item.bytes, bytes_length);

	return *this;
}
// codeが一致するか
bool CodeMapItem::CmpCode(long code_name) {
	if (code == code_name) return true;
	else return false;
}
// strが一致するか
bool CodeMapItem::CmpStr(const wxString &str_name) {
	if (str == str_name) return true;
	else return false;
}
// strに指定した文字列が含まれるか(前方一致)
bool CodeMapItem::FindStr(const wxString &str_name, bool case_insensitive) {
	int pos = 0;
	if (case_insensitive) {
		pos = str_name.Find(str_upper);
	} else {
		pos = str_name.Find(str);
	}
	if (pos == 0) return true;
	else return false;
}
// bytesに指定したバイト列が含まれるか(前方一致)
bool CodeMapItem::FindBytes(const wxUint8 *bytes_name) {
	char *pos = strstr((char *)bytes_name, (const char *)bytes);
	if (pos != NULL && pos == (char *)bytes_name) return true;
	return false;
}
// attrに指定した文字列が含まれるか
bool CodeMapItem::FindAttr(const wxString &attr_name) {
	if (attr.Find(attr_name) != wxNOT_FOUND) return true;
	else return false;
}

// マッピングテーブルItemArray
WX_DEFINE_OBJARRAY(CodeMapItems);

// マッピングテーブルSection
CodeMapSection::CodeMapSection(const wxString &new_name) {
	name = new_name;
	items.Empty();
}
bool CodeMapSection::CmpSection(const wxString &section_name) {
	if (name == section_name) return true;
	else return false;
}
// アイテムを追加
void CodeMapSection::AddItem(const CodeMapItem *new_item) {
	items.Add(new_item);
}
void CodeMapSection::AddItem(const long &new_code, const wxString &new_str, const wxString &new_attr) {
	CodeMapItem *new_item = new CodeMapItem(new_code, new_str, new_attr);
	AddItem(new_item);
}
// アイテムを探す(code)
CodeMapItem *CodeMapSection::FindByCode(long code, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	size_t i;
	for (i = 0; i < items.GetCount(); i++) {
		if (items.Item(i).CmpCode(code)) {
			item = &(items.Item(i));
			if (attr.IsEmpty() != true) {
				bool match = item->FindAttr(attr);
				if (match == matching) {
					break;
				} else {
					item = NULL;
				}
			} else {
				break;
			}
		}
	}
	return item;
}
// アイテムを探す(str)(前方一致 & 最長一致)
CodeMapItem *CodeMapSection::FindByStr(const wxString &str, bool case_insensitive, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	CodeMapItem *item_max = NULL;
	size_t i;
	size_t len_max = 0;
	bool find = false;
	for (i = 0; i < items.GetCount(); i++) {
		find = false;
		if (items.Item(i).FindStr(str, case_insensitive)) {
			find = true;
			item = &(items.Item(i));
			if (attr.IsEmpty() != true) {
				bool match = item->FindAttr(attr);
				if (match != matching) {
					find = false;
				}
			}
			if (find) {
				if(item->str.Len() > len_max) {
					item_max = item;
					len_max = item->str.Len();
				}
			}
		}
	}
	return item_max;
}
// アイテムを探す(bytes)(前方一致 & 最長一致)
CodeMapItem *CodeMapSection::FindByBytes(const wxUint8 *bytes, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	CodeMapItem *item_max = NULL;
	size_t i;
	size_t len_max = 0;
	bool find = false;
	for (i = 0; i < items.GetCount(); i++) {
		find = false;
		if (items.Item(i).FindBytes(bytes)) {
			find = true;
			item = &(items.Item(i));
			if (attr.IsEmpty() != true) {
				bool match = item->FindAttr(attr);
				if (match != matching) {
					find = false;
				}
			}
			if (find) {
				if(item->bytes_length > len_max) {
					item_max = item;
					len_max = item->bytes_length;
				}
			}
		}
	}
	return item_max;
}

// マッピングテーブルSectionArray
WX_DEFINE_OBJARRAY(CodeMapSections);

// マッピングテーブル本体
CodeMapTable::CodeMapTable() {
	sections.Empty();
	current_section = NULL;
}
// セクションを追加
void CodeMapTable::AddSection(const wxString &section_name) {
	// 重複を避ける
	if (!FindSection(section_name)) {
		CodeMapSection new_section(section_name);
		sections.Add(new_section);
		current_section = &(sections.Last());
	}
}
// セクションを探す
bool CodeMapTable::FindSection(const wxString &section_name) {
	bool find = false;
	size_t i;
	for (i = 0; i < sections.GetCount(); i++) {
		if (sections.Item(i).CmpSection(section_name)) {
			find = true;
			break;
		}
	}
	if (find) {
		current_section = &(sections.Item(i));
	}
	return find;
}
// 現在のセクションを返す
CodeMapSection *CodeMapTable::GetCurrentSection() {
	return current_section;
}
// セクション名を返す
void CodeMapTable::GetAllSectionNames(wxArrayString &section_names) {
	size_t i;
	for (i = 0; i < sections.GetCount(); i++) {
		section_names.Add(sections.Item(i).name);
	}
	return;
}
wxString &CodeMapTable::GetSectionName(size_t index) {
	return sections.Item(index).name;
}

// アイテムを追加
void CodeMapTable::AddItem(const long &new_code, const wxString &new_str, const wxString &new_attr) {
	if (current_section != NULL) current_section->AddItem(new_code, new_str, new_attr);
}
// アイテムを探す(code)
CodeMapItem *CodeMapTable::FindByCode(long code, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	if (current_section != NULL) item = current_section->FindByCode(code, attr, matching);
	return item;
}
// アイテムを探す(str)(前方一致 & 最長一致)
CodeMapItem *CodeMapTable::FindByStr(const wxString &str, bool case_insensitive, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	wxString n_str;
	if (case_insensitive) n_str = str.Upper(); else n_str = str;
	if (current_section != NULL) item = current_section->FindByStr(n_str, case_insensitive, attr, matching);
	return item;
}
// アイテムを探す(bytes)(前方一致 & 最長一致)
CodeMapItem *CodeMapTable::FindByBytes(const wxUint8 *bytes, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	if (current_section != NULL) item = current_section->FindByBytes(bytes, attr, matching);
	return item;
}
// 全セクションでアイテムを探す(code)
CodeMapItem *CodeMapTable::FindByCodeInAllSections(long code, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	size_t i;
	for (i = 0; i < sections.GetCount(); i++) {
		item = sections.Item(i).FindByCode(code, attr, matching);
		if (item != NULL) break;
	}
	return item;
}
// 全セクションでアイテムを探す(str)(前方一致 & 最長一致)
CodeMapItem *CodeMapTable::FindByStrInAllSections(const wxString &str, bool case_insensitive, const wxString &attr, bool matching) {
	CodeMapItem *item = NULL;
	wxString n_str;
	if (case_insensitive) n_str = str.Upper(); else n_str = str;
	size_t i;
	for (i = 0; i < sections.GetCount(); i++) {
		item = sections.Item(i).FindByStr(str, case_insensitive, attr, matching);
		if (item != NULL) break;
	}
	return item;
}
