//
// parse.h
//
#ifndef _PARSE_H_
#define _PARSE_H_

#include "common.h"
#include "wx/wx.h"
#include "wx/file.h"
#include "wx/datstrm.h"
#include "wx/dynarray.h"
#include "errorinfo.h"
#include "maptable.h"
#include "fileinfo.h"
#include "parseresult.h"

// p[T[NX
class Parse
{
private:
	wxString mAppPath;

	PsErrInfo mErrInfo;	// G[

	PsFileInfo mInFile;		// ̓t@C
	PsFileInfo mOutFile;		// o̓t@C

	int mNextAddress;		// AhX

	PsFileType     mParsedType;
	wxArrayString mParsedData;	// ʕ\pf[^obt@

	wxString mAsciiNewLine;	// sR[h
	wxString mUtf8NewLine;	// sR[h

	CodeMapTable mCharCodeTbl;	// R[hϊe[u
	CodeMapTable mBasicCodeTbl;	// BASICR[hϊe[u

	// p[^
	int mNewLineAscii;
	int mNewLineUtf8;
	bool mIncludeBOM;
	int  mStartAddr;

	// ̓f[^̃tH[}bg`FbN
	bool CheckDataFormat(PsFileInfo &in_file_info);
	// Ԍ`f[^̃tH[}bg`FbN
	bool CheckBinaryDataFormat(PsFileInfo &in_file_info);
	// AXL[`f[^̃tH[}bg`FbN
	bool CheckAsciiDataFormat(PsFileInfo &in_file_info, size_t start_pos = 0);
	// Ԍ`f[^̓ǂݒ
	bool ReloadBinaryData(PsFileInfo &in_file_info, PsFileType &out_type);
	// AXL[`f[^̓ǂݒ
	bool ReloadAsciiData(PsFileInfo &in_file_info, PsFileType &out_type);
	bool ReloadAsciiData(PsFileInfo &in_file_info, PsFileType &out_type, size_t start_pos);
	// ԌꂩAXL[`eLXgɕϊ
	bool ReadBinaryToAscii(PsFileInfo &in_file, PsFileType &out_file_type, wxArrayString &outlines, ParseResult *result = NULL);
	// AXL[`璆Ԍɕϊ
	bool ParseAsciiToBinary(PsFileType &in_file_type, wxArrayString &inlines, PsFileInfo &out_file, ParseResult *result = NULL);
	// AXL[`1s𒆊Ԍɕϊ
	bool ParseAsciiToBinaryOneLine(PsFileType &in_file_type, wxString &in_data, PsFileInfo &out_file, int row = 0, ParseResult *result = NULL);
	// AXL[`eLXgǂ
	bool ReadAsciiText(PsFileInfo &in_file, wxArrayString &outlines, bool to_utf8 = false);
	// eLXgo
	bool WriteText(wxArrayString &inlines, PsFileInfo &out_file);
	// AXL[`UTF-8eLXgɕϊ
	bool ConvAsciiToUTF8(wxArrayString &inlines, PsFileType &out_file_type, wxArrayString *outlines, ParseResult *result = NULL);
	// UTF-8eLXgAXL[`ɕϊ
	bool ConvUTF8ToAscii(PsFileType &in_file_type, wxArrayString &inlines, wxArrayString *outlines, ParseResult *result = NULL);
	// R[hϊe[u̓ǂݍ
	PsErrType LoadCharCodeTable();
	// BASICR[hϊe[u̓ǂݍ
	PsErrType LoadBasicCodeTable();

public:
	Parse(const wxString &path);
	~Parse();
	bool Init();
	// w肵t@CJ
	bool OpenDataFile(const wxString &in_file_name);
	// t@C
	void CloseDataFile();
	// t@CJĂ邩
	bool IsOpenedDataFile();
	// JĂt@C
//	int GetOpenedDataType();
	PsFileType *GetOpenedDataTypePtr();
	// t@C
	wxString GetFileNameBase();
	// o̓t@CJ
	bool OpenOutFile(const wxString &out_file_name, PsFileType &file_type);
	// o̓t@C
	void CloseOutFile();
	// GNX|[g
	bool ExportData();
	// ʕ\pf[^Ԃ
	wxString &GetParsedData(wxArrayString &lines);
	// ʕ\pf[^
//	int GetParsedDataType();
	PsFileType *GetParsedDataTypePtr();
	// ނԂ
	void GetCharTypes(wxArrayString &char_types);
	wxString &GetCharType(size_t index);
	// BASICނԂ
	void GetBasicTypes(wxArrayString &basic_types);
//	// BASICނZbg
//	void SetBasicType(const wxString &basic_type);
	// ̓t@CBASICނԂ
	wxString &GetOpenedBasicType();
	// ̓t@Č`ύXēǂݒ
	bool ReloadOpendAsciiData(int type, int mask, const wxString &char_type);
	// \pf[^̌`ύXēǂݒ
	bool ReloadParsedData(int type, int mask, const wxString &char_type);
	// Ԍ`f[^̓ǂݒ
	bool ReloadOpenedBinaryData(int type, int mask, const wxString &basic_type);
	// X^[gAhX̃Xgւ̃|C^Ԃ
	int *GetStartAddrsPtr(size_t *count);
	// sԍԂ
	long GetLineNumber(const wxString &line, size_t *next_pos = NULL);
	// |[g
	void Report(ParseResult &result, wxArrayString &line);

	// p[^
	void SetNewLineAscii(int val) { mNewLineAscii = val; }
	void SetNewLineUtf8(int val) { mNewLineUtf8 = val; }
	void SetIncludeBOM(bool val) { mIncludeBOM = val; }
	void SetStartAddr(int val) { mStartAddr = val; }
	int GetNewLineAscii() { return mNewLineAscii; }
	int GetNewLineUtf8() { return mNewLineUtf8; }
	bool GetIncludeBOM() { return mIncludeBOM; }
	int GetStartAddr() { return mStartAddr; }

};

#endif /* _PARSE_H_ */
